function calmat= calib_algo1_pca(data, opts)
% Computes the matrices necessary for the affine transformation 
% that maps the recorded activity to a 2D force vector
%
% INPUTS
% - data: (time X Trials X nUnits X nStimuli) matrix containing spike count
% - opts: contains dynamical system parameters 
%
% OUTPUTS
% - meanData: mean of training set across trials (phi)
% - invPHI: inverse of PHI matrix
% - MeanD: mean of d-vectors matrix
% - ScaleMatrix: transformation matrix
%__________________________________________________________________________

% -------------------------
% load dynamical system parameters
% -------------------------
dynSys_opts = dynSys_opts_REPOSITORY(opts.dynSys);
Stiffness = dynSys_opts.stiffness;
Xrange = dynSys_opts.Xrange;
Yrange = dynSys_opts.Yrange;
% calculate force range
[FxR, FyR]= rangeF(Stiffness, Xrange, Yrange);
% -------------------------

% get data dimensions
[nT, nTrial, nUnit, nStim] = size(data);

% compute phi, the average response to each stim
meanData = zeros(nT, nUnit,nStim);  % needed in case nUnit = 1
meanData(:,:,:) = squeeze(nanmean(data,2)); % phi

% Build the matrix PHI = <phi, phi>
PHI = zeros(nStim);
for ii = 1:nStim
    for jj = 1:nStim
         PHI(ii,jj) = sum(sum(squeeze(meanData(:,:,ii)).*squeeze(meanData(:,:,jj))));
    end
end
invPHI = pinv(PHI);

% Project all the calibration responses on the mean calibration responses 
% PSI(r,s,i)= < phi(i), psi(r,s) >
TmpData = permute(data,[1 3 2 4]);
TmpData = reshape(TmpData, nT, nUnit, nTrial*nStim);
TmpData = repmat(TmpData, [1, 1, 1, nStim]);
TmpMeanData = repmat(meanData, [1, 1, 1, nTrial*nStim]);
TmpMeanData = permute(TmpMeanData,[1 2 4 3]);
PSI = squeeze(sum(sum(TmpMeanData.*TmpData,1),2));

% Collect all the S-dimensional d- vectors in a data matrix X
X = (invPHI*PSI')'; 
MeanD = mean(X);

% Calculate the PCs over X
% pc = princomp(X)';
pc = princomp(zscore(X))';
% [COEFF,SCORE,latent,tsquare]= princomp(zscore(X));
% The 2 leading PCs are collected in a projection operator W
W = [pc(1,:);pc(2,:)];  

% Map to 2D by PCA
xydata = W*detrend(X,0)'; % detrend(X,0) removes mean across column (meanD)

% find maximum variation along the two dimensions
DX = abs(max(xydata(1,:)) - min(xydata(1,:)));
DY = abs(max(xydata(2,:)) - min(xydata(2,:)));

% Compute the scaling matrix
scale(1,1)= FxR/DX;
scale(2,2)= FyR/DY;
scale(1,2)= 0.0; 
scale(2,1) = 0.0;

% And derive the net transformation matrix
ScaleMatrix = scale*W;


%____________________
%       SAVE 
%____________________
calmat.algo='algo1';
calmat.nStim=nStim;
calmat.meanData=meanData;
calmat.invPHI=invPHI;
calmat.MeanD=MeanD;
calmat.ScaleMatrix=ScaleMatrix;


% Compute the force corresponding to each stimulus
calmat.RecForce=zeros(2,nStim);
for kk = 1:nStim
    activity = zeros(nT, nUnit);    % needed in case nT == 1
    activity(:,:) = squeeze(meanData(:,:,kk));
    Force=comp_force_algo(calmat, activity,opts);
    calmat.RecForce(:,kk) = Force.Force;
end

% derive the corresponding recPoints
calmat.RecPoint = -inv(Stiffness)*calmat.RecForce;
