function sim = runSimulation(RecPoint, ForceTest, trials, opts)


global ExternalForce Mass Damping % parameters to be shared with MBF

% -------------------------
% load dynamical system parameters
% -------------------------
dynSys_opts = dynSys_opts_REPOSITORY(opts.dynSys);
% Stiffness = dynSys_opts.stiffness;
Mass = dynSys_opts.mass;
Damping = dynSys_opts.damping;
Xrange = dynSys_opts.Xrange;
Yrange = dynSys_opts.Yrange;
Radius = dynSys_opts.radius;
Zero = dynSys_opts.zero;
% -------------------------         

% -------------------------
% load simulation parameters
% -------------------------
sim_opts = sim_opts_REPOSITORY(opts.sim);
nRun = sim_opts.nRun;
runTime = sim_opts.runTime;
totalTime = sim_opts.totalTime;
% -------------------------

% number of time steps per run
nTicks = round(totalTime/runTime); 

% init matrices
dataState = zeros(nRun, nTicks, 4);
dataForce = zeros(nRun, nTicks, 2);
dataStimulation = zeros(nRun, nTicks);
% dataActivity = zeros(nRun, nTicks, nT, nUnit);
dataConverge = NaN(nRun,1);
TriFail = 0;

% triangulation
try
    %%%%%%% delaunay updated%%%%%%%%%%%%%%%%%%%%%%%%
%     TRI = delaunay(RecPoint(1,:),RecPoint(2,:)); 
    TRI = DelaunayTri(RecPoint(1,:)',RecPoint(2,:)'); 
catch
    TriFail = 1;
    disp('no TRI');
end

% if the triangulation didn't fail, run the simulation
if ~TriFail
    
    for ii = 1:nRun
        
        xlimit = sum(abs(Xrange));
        ylimit = sum(abs(Yrange));
        
        % if starting positions have been defined in the options of
        % simulation then load them
        if isfield(sim_opts,'X0')
            xi = Xrange(2)*sim_opts.X0(1,ii);
            yi = Yrange(2)*sim_opts.X0(2,ii);
        else
            % pick one point at random
            xi = rand*xlimit - xlimit/2;
            yi = rand*ylimit - ylimit/2;
            % check it is not too close
            while sqrt(xi^2 + yi^2) < sqrt((Xrange(2)/3)^2 + (Yrange(2)/3)^2)
                xi = rand*xlimit - xlimit/2;
                yi = rand*ylimit - ylimit/2;
            end
        end;
        
        % Set it as the initial state
        State = [xi yi 0 0];  % Start at rest
        
%         %%%% ROBOTIC ARM %%%%%%
%         %   Define arm parameters
%         initParams;
%         % define initial state in angular coordinates
%         StateAng = [inverseKinematics([xi yi+ylimit/2]) 0 0]; %shoulder is at the bottom of the field
%         %%%% ROBOTIC ARM %%%%%%
        
        for jj = 1:nTicks
            if indeadzone(State, Radius, Zero)
                break;
            else
                
                % find to which calibration point the point mass is, in
                % order to apply the corresponding stimulus
                % (WARNING: replace by 'nearestNeighbor')
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% updated%%%%%%%%%%%%%%%%%%%%%%%%
%                 nearestStim = dsearch(RecPoint(1,:), RecPoint(2,:), TRI, State(1), State(2));
                nearestStim = nearestNeighbor(TRI,State(1), State(2));
                
                % stimulate and record
                index = ceil(rand*trials(nearestStim));

                % get the corresponding force
                Force=squeeze(ForceTest(index,nearestStim,:));

                % apply the force to the point mass
                ExternalForce = - Force;
                
                % WARNING: the time argument isn't used when solving the
                % differential equation
                [time, StateVector] = ode45(@MBF, [0, runTime], State);
                State=StateVector(end,:);
                
%                 %%%% ROBOTIC ARM %%%%%%
%                 [time, StateVector] = ode45(@MBFarm, [0, runTime], StateAng);
%                 %                 StateVector=JointToTip(StateVector);
%                 StateAng = StateVector(end,:)';
%                 [tip,elbow]=forwardKinematics(StateAng(1:2)');
%                 State(1)=tip(1); 
%                 State(2)=tip(2)-ylimit/2; % back to FF coordinates
%                 %%%% ROBOTIC ARM %%%%%%
                
                % record data
                dataState(ii, jj, :) = State;
                dataForce(ii, jj, :) = Force;
                dataStimulation(ii, jj) = nearestStim;
%                 dataActivity(ii, jj, :, :) = activity;
            end
        end
        
        if indeadzone(State, Radius, Zero)
            dataConverge(ii) = jj;
        end
        
    end
end


sim.State = dataState;
sim.Force = dataForce;
sim.Stimulation = dataStimulation;
%sim.Activity = dataActivity;
sim.Converge = dataConverge;
sim.TriFail = TriFail;
