function [D distmat]=compDist(spkmat,opts)
% Multineuron Spike Train Metric described in (Houghton & Sen, 2008)
% - implementation by Francois Szymanski -
%
% spkmat : spike trains [time (1kHz) x trial x Single Unit x stimulus]
%
% opts.trialN : number of trial for each stimulus [1 nstim]
%
% opts.tc : time constant of the decaying exponentional kernel (default=10ms)
%
% opts.teta : angle between unit vectors.
% teta = 0: summed population metric
% teta = pi/2 : labelled line metric


if isfield(opts,'saveitonhdd') && opts.saveitonhdd==1
    saveitonhdd=1;
    [dirName, fileName] = file_opts_REPOSITORY(opts, 'disttmp');
    existinhdd=exist(fullfile(dirName, [fileName '.mat']),'file')==2;
else
    saveitonhdd=0;
end;

if saveitonhdd && existinhdd
    load(fullfile(dirName, fileName));
    D=dist.D;
    distmat=dist.distmat;
    
else
    
    trialN = opts.trialN;
    
    [nt maxtr nunit nstim] = size(spkmat);
    
    dist_opts = dist_opts_REPOSITORY(opts.dist);
    tc=dist_opts.tc;
    teta=dist_opts.teta;
    
    t=0:1:4*tc;
    K = (1/sqrt(tc))*exp(-t/tc);
    K2D=zeros(nt,nt+length(K)-1,'single');
    for tt=1:nt,
        K2D(tt,tt:tt+length(K)-1)=K;
    end;
    
    % mapping of the spike trains to real functions with the kernel K
    A=zeros(nt+length(K)-1, sum(trialN), nunit,'single');
    offset=0;
    for st=1:nstim,
        for su=1:nunit,
            for tr=1:trialN(st),
                A(:,offset+tr,su)=sum(K2D(squeeze(spkmat(:,tr,su,st))>0,:),1);
            end;
        end
        offset=offset+trialN(st);
    end;
    A=A(1:nt,:,:);
    
    % compute unit vectors
    e=zeros(nunit,'single');
    if teta>0 % to avoid the division by zero...
        e(1)=1;
        
        if nunit >=2
            costeta=cos(teta);
            sinteta=sin(teta);
            e(1,2:end)=costeta;
            e(2,2)=sinteta;
        end;
        
        for uu=3:nunit,
            costeta=(costeta-costeta^2)/sinteta^2;
            sinteta=sqrt(1-costeta^2);
            e(uu-1,uu:end)=e(uu-1,uu-1)*costeta;
            e(uu,uu)=e(uu-1,uu-1)*sinteta;
        end
    else
        e(1,:)=1;
    end;
    % project each SU response onto its unit vector (mapping to vector field)
    % and summation of the vector fields associated to each neuron
    Anew=zeros(size(A),'single');
    % for uu=1:nunit,
    %     Atmp=repmat(A(:,:,uu),[1 1 nunit]);
    % %     etmp=shiftdim(repmat(e(:,uu),[1 nt+length(K)-1 sum(trialN)]),1); % out of memory
    %     etmp0=zeros(1,1,nunit);
    %     etmp0(1,1,:)=e(:,uu);
    %     etmp=repmat(etmp0,[nt+length(K)-1 sum(trialN) 1]);
    %     Anew=Anew+Atmp.*etmp;
    % end;
    for dd=1:nunit
        for uu=1:nunit,
            Anew(:,:,dd)=squeeze(Anew(:,:,dd))+e(dd,uu)*A(:,:,uu);
        end;
    end;
    
    % compute distance between each pair of vector field
    p=2;
    D=zeros(sum(trialN),'single');
    
    % tic
    for ii=1:sum(trialN)
%         for jj=1:ii-1
%             D(ii,jj)=(sum(sum((Anew(:,ii,:)-Anew(:,jj,:)).^p,3),1)).^(1/p);
%         end;
        
            %85min instead of 94min for an example data set
            jj=1:ii-1;
            D(ii,jj)=(sum(sum((repmat(Anew(:,ii,:),[1 length(jj) 1])-Anew(:,jj,:)).^p,3),1)).^(1/p);
    end;
    % toc
    
    
    D=D+D';
    
    distmat.K2D=K2D;
    distmat.e=e;
    distmat.Acal=Anew;
end;


if saveitonhdd && ~existinhdd
    dist.set=opts.set;
    dist.trialN=trialN;
    dist.teta=teta;
    dist.tc=tc;
    dist.D = D;
    dist.distmat=distmat;
    save(fullfile(dirName, fileName), 'dist');
end;
