function [data, dataParams] = createExpMatrix(filename)
% function createExpMatrix organizes spike count data sampled at 1kHz of different neurons responding to different stimulation events in one matrix 
%
% filename: name of .plx file
% 
% data: (time X maxTrials X nUnits X nStimuli) matrix containing spike count
% dataParams: 3 elemets structure:
%               - unitIndex: (2 X nUnits) matrix each columns contains channel and unit number of active units
%               - trialN: vector of nStimuli elements containing number of trials of each stimulus
%               - stimPatterns: matrix of stimulation pattern (each row represents a pattern)

% Marianna Semprini & Franois Szymanski
% IIT, April 2010

[ts, wf, ev] = plx_info(filename, 1); % extract number of recorded spikes (ts) and stimulation events (ev)
clear wf;

% find number and index of active units
[r, c] = size(ts);
[u, ch] = find(ts(2:r,:));  % returning the index of active unit
ch = ch-1;                  % as first row is unsorted data
unitIndex = [ch'; u'];
plx_close(filename);

% find number of active event
activeEvt = find(ev);                       % returning the index of existing events
index = find(activeEvt>2 & activeEvt<10);   % stimulation events
stimIndex = activeEvt(index);               % returning the index of stimulation events
stimEvt = cell(1,numel(stimIndex));         % will contain time stamps of each event
timeMax= zeros(1,length(stimIndex));
for ii = 1:numel(stimIndex)
    [n ts] = plx_event_ts(filename, stimIndex(ii)); % find timestamps of event ii
    stimEvt(ii) = {ts};                             % event time stamps
    timeMax(ii) = ts(end);                          % last time stamp of event ii
end

res = 1e-2; % resolution in seconds
timeMax = ceil(max(timeMax)/res);               % find last event time stamp and convert according to res
EvtMatrix = zeros(timeMax,length(stimIndex));   % will contain event type
TSMatrix = zeros(timeMax,length(stimIndex));    % will contain time stamps
for ii = 1:length(stimIndex), 
    tmpSpkIdx = ceil(stimEvt{ii}/res);      % convert time stamps to indices 
    EvtMatrix(tmpSpkIdx,ii) = 1;            % active event
    TSMatrix(tmpSpkIdx,ii) = stimEvt{ii};   % store true time stamps
end;
TSMatrix(TSMatrix==0) = NaN;
TSMatrix = nanmean(TSMatrix,2);

stimPatterns = unique(EvtMatrix,'rows');            % find all combination of events that correspond to stimulation pattern
stimPatterns(find(sum(stimPatterns,2)==0),:) = [];  % remove the case with no events
nStims = size(stimPatterns,1);                      % number of stimulation patterns
stimEvt2 = cell(1,numel(nStims));                   % will contain time stamps of each stimulation pattern
trialN = zeros(1,nStims);                           % will contain number of trials of each stimulation pattern
for ii = 1:nStims,
    tmp = EvtMatrix == repmat(stimPatterns(ii,:),timeMax,1);    % find matching of EvtMatrix with stimulation pattern ii
    tmpEvtIdx = find(sum(tmp,2)==length(stimIndex));            % find indices of stimulation pattern ii (correspond to perfect matching)
    stimEvt2(ii) = {TSMatrix(tmpEvtIdx)};                       % time stamps of stimulation ii
    trialN(ii) = length(tmpEvtIdx);                             % number of trials of stimulation ii    
end;

MIN = 0;    % start time (ms) of data collection after stimulation
MAX = 1000; % end time (ms) of data collection after stimulation
 
data = NaN(MAX, max(trialN), numel(unitIndex)/2, nStims, 'single'); % 'single' precision make data less heavy fo the memory

for ii = 1:numel(unitIndex)/2
    [n, ts] = plx_ts(filename,unitIndex(1,ii),unitIndex(2,ii)); % extract timestamps of spikes of unit ii
    
    for jj = 1:nStims
        stimTS = stimEvt2{jj};   % extract timestamps of stimulation pattern jj
        
        for kk = 1:trialN(jj)           
            trialTS = stimTS(kk);
            minTs = trialTS + MIN/1000;         % define mininum value for observation window (converting to seconds)
            maxTs = trialTS + (MAX-1)/1000;     % define maximum value for observation window (converting to seconds)
            dataTmp = ts(find(ts>=minTs & ts<=maxTs)) - trialTS; % considering recorded data only in that window
            data(:,kk,ii,jj) = 0;               % set recorded activity to 0 everywhere
            if ~isempty(dataTmp)
                data(round(1000*dataTmp)+1,kk,ii,jj) = 1; % set recorded activity to 1 in timestamps corresponding to spikes 
            end
        end
    end
end

plx_close(filename);

dataParams.unitIndex = unitIndex;
dataParams.trialN = trialN;
dataParams.stimPatterns = stimPatterns;
  